//-< RawBinaryFactory.java >-----------------------------------------*--------*
// GOODS                      Version 2.02       (c) 1998  GARRET    *     ?  *
// (Generic Object Oriented Database System)                         *   /\|  *
// Java Program Interface                                            *  /  \  *
//                          Created:     26-Sep-2002  K.A. Knizhnik  * / [] \ *
//                          Last update: 26-Oct-2002  K.A. Knizhnik  * GARRET *
//-------------------------------------------------------------------*--------*
// Interace of factory for packing/unpacking raw binary objects
//-------------------------------------------------------------------*--------*
namespace Goods
{
    using System;
	
    /// <summary> Interace of factory for packing/unpacking raw binary objects
    /// </summary>
    public interface RawBinaryFactory
    {
        /// <summary> Get size of raw binary object in bytes
        /// </summary>
        /// <param name="obj">raw binary object
        /// </param>
        /// <returns>size of packed raw binary object
        /// 
        /// </returns>
        int size(Object obj);
        /// <summary> Pack raw binary object to byte array
        /// </summary>
        /// <param name="obj">raw binary object
        /// </param>
        /// <param name="arr">array to store packed image of raw binary object
        /// </param>
        /// <param name="off">offset in the array where object should be packed
        /// </param>
        /// <returns>numbumer of bytes in packed object image.
        /// It should be the same as returned by <code>RawBinary.size</code> method.
        /// 
        /// </returns>
        int pack(Object obj, byte[] arr, int off);
        /// <summary> Unpack raw binary object
        /// </summary>
        /// <param name="cls">class of raw binary object
        /// </param>
        /// <param name="arr">array containing packed raw binary object
        /// </param>
        /// <param name="off">offset of packed binary object in the array
        /// </param>
        /// <param name="size">size of packed binary object
        /// </param>
        /// <returns>created raw binary object with unpacked content
        /// 
        /// </returns>
        Object unpack(Type cls, byte[] arr, int off, int size);
        /// <summary> Check if specified class supported by this factory
        /// </summary>
        /// <param name="cls">raw binary object class
        /// </param>
        /// <returns><code>true</code> if factory is able to pack/unpack instances of these class
        /// 
        /// </returns>
        bool supports(Type cls);
    }
	
}