//-< TESTSOCK.CXX >--------------------------------------------------*--------*
// GOODS                     Version 1.0         (c) 1997  GARRET    *     ?  *
// (Generic Object Oriented Database System)                         *   /\|  *
//                                                                   *  /  \  *
//                          Created:     24-Oct-97    K.A. Knizhnik  * / [] \ *
//                          Last update: 24-Oct-97    K.A. Knizhnik  * GARRET *
//-------------------------------------------------------------------*--------*
// Program for testing sockets performance
//-------------------------------------------------------------------*--------*

#include "goods.h"
#include "sockio.h"
#include <time.h>

USE_GOODS_NAMESPACE

#define HDR_SIZE  16
#define BODY_SIZE 240
#define N_LOOPS   10000000


event completed;

void task_proc local_server(void* address) 
{ 
    int i;
    msg_buf errbuf;
    char    buf[HDR_SIZE+BODY_SIZE];    
    socket_t* gate = socket_t::create_local((char*)address); 
    if (!gate->is_ok()) { 
        gate->get_error_text(errbuf, sizeof errbuf);
        console::error("Failed to create local socket: %s\n", errbuf);
    }
    socket_t* sock = gate->accept();
    if (sock == NULL) { 
        gate->get_error_text(errbuf, sizeof errbuf);
        console::error("Failed to accept socket: %s\n", errbuf);
    }
    delete gate;
    
    for (i = 0; i < N_LOOPS; i++) { 
        if (!sock->read(buf, HDR_SIZE) ||
            !sock->read(buf, sizeof(buf) - HDR_SIZE) ||
            !sock->write(buf, sizeof buf)) 
        { 
            sock->get_error_text(errbuf, sizeof errbuf);
            console::error("Socket operation failed: %s\n", errbuf);
        }
    }   
    console::output("Receive %d requests\n", i);
    delete sock;
    completed.signal();
}

void task_proc local_client(void* address) 
{
    int i;
    msg_buf errbuf;
    char    buf[HDR_SIZE+BODY_SIZE];    
    socket_t* sock = socket_t::connect((char*)address); 
    if (sock == NULL) { 
        console::error("Failed to connect to server\n");
    } else if (!sock->is_ok()) { 
        sock->get_error_text(errbuf, sizeof errbuf);
        console::error("Connection to server failed: %s\n", errbuf);
    }
    for (i = 0; i < N_LOOPS; i++) { 
        if (!sock->write(buf, sizeof buf) ||
            !sock->read(buf, HDR_SIZE) ||
            !sock->read(buf, sizeof(buf) - HDR_SIZE))
        {
            sock->get_error_text(errbuf, sizeof errbuf);
            console::error("Socket operation failed: %s\n", errbuf);
        }
    }
    delete sock;
    console::output("Send %d requests\n", i);
}

int main(int argc, char* argv[]) 
{
    task::initialize(task::huge_stack);

    if (argc < 2) { 
        console::output("Socket test program\n"
                        "Usage: testsock2 host:port\n");
        return EXIT_FAILURE;
    }

    time_t start_time = time(NULL);
    task::create(local_server, argv[1]);
    task::create(local_client, argv[1]);
    completed.wait();
    
    console::output("Elapsed time %d seconds\n", time(NULL) - start_time);
    return EXIT_SUCCESS;
}



