package org.dcg.datamgr.databasejobs;

import org.dcg.datamgr.goodsindex.*;

import goodsjpi.Database;

/**
 * @author dacosta
 *
 * This Class has to be used for any DATABASE RELATED task to be executed by a PoolMgr.<br>
 * The basic functionality takes place in the task2bExecuted() method.<br>
 * Access is gained through the run() statement (which is also the ONLY public method available.<br>
 * Any other additional method HAS to be private for they are only allowed to HELP the main task.<br>
 * 
 * Data going in is set via the constructor.<br>
 * Data going out goes through a non-final object entering via the run(xyz).<br>
 */
public abstract class DatabaseJob implements java.lang.Runnable {
    protected Database _db;
	protected RootClass _root;
	protected Object _data;
		
	public void run() {
		System.err.println("This method is NOT implemented.\nUse run(Database, RootClass) instead");
	}
	
	public void run(Database db, RootClass root) {
		_db=db;
		_root=root;
		_db.attach();
		task2bExecuted();
		_db.detach();
	}
	
	/**
	 * Actual tasks to be executed are to be inserted under this method.<br>
	 * Stuff to be synchronized should be handled here.<br>
	 * As well as the begin/end bulk transaction stuff.<br>
	 * There are basic read and write bulk-write classes available though.<br>
	 */
	protected abstract void task2bExecuted();
	
	public abstract void setDatabaseParameters();
	
}
