package org.dcg.datamgr.goodsindex;


import java.util.Collections;
import java.util.Map;
import java.util.Set;

import goodsjpi.Metaobject;
import goodsjpi.OptimisticMetaobject;
import goodsjpi.PessimisticMetaobject;
import goodsjpi.Persistent;
import goodsjpi.PessimisticRepeatableReadMetaobject;
import goodslib.HashTable;
import goodslib.PersistentHashMap;
import goodslib.PersistentHashSet;
import goodslib.PersistentTreeMap;
import goodslib.PersistentTreeSet;

public class RootClass extends Persistent {
	final static Metaobject defaultMetaObject = new OptimisticMetaobject(); 
    
	public Map _classObjects;
	public Map _primitives;
    public Map classIndexMap;
    private Map _primitiveValues;

    public RootClass() {
        classIndexMap = new PersistentTreeMap();
        _primitives = new PersistentHashMap();
		_classObjects = new PersistentHashMap();
		_primitiveValues = new PersistentTreeMap();
	}

	/**
	 * Returns the classObjects. Basically these are empty instances of the classes which can be persisted in the database.</ br>
	 * Rather then filling the memory these are persisted. Might be slower but GC is worse than a simple disk-read.</ br>
	 * @return Map with <em>full classname = classinstance(/object)</em>
	 */
	public Map getClassObjects() {
		return _classObjects;
	}

	/**
	 * Returns the primitives (attached to their classes) which can be found in the database.</ br>
	 * @return PersistentHashMap 
	 */
	public Map getPrimitives() {
		return _primitives;
	}

	/**
	 * Returns the Map containing the primitiveValues and the corresponding instances within the database.</ br>
	 * @return Map with the key-set (with instances)
	 */
	public Map getPrimitiveValues() {
		return _primitiveValues;
	}

	public void setPrimitiveValues(final String key, final Object value) {
		((Set)_primitiveValues.get(key)).add(value);
	}

	public void setNewValueIntoPrimitiveValues (final String key, final Object value) {
		if (! _primitiveValues.containsKey(key)) _primitiveValues.put(key, new PersistentTreeSet());
		setPrimitiveValues(key, value);
	}
	
	/**
	 * Returns the classIndexMap.
	 * @return ClassIndexMap
	 */
	public Map getClassIndexMap() {
		return classIndexMap;
	}

}
